#  R-Version:    3.1.1 (2014-07-10)
#
#   Changes in "by" function as compared to previous
#       R-versions and scripts
#
#  Required input files:
#
# Two files contain the dataset used in the analysis and can
#  be obtained from GEO:
#
#    GSE31519_complete_dataset.txt 
#    GSE31519_TNBC_SampleInfo_BCR.txt 
#   
#
# GSE31519_TNBC_SampleInfo_BCR.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5FTNBC%5FSampleInfo%5FBCR%2Etxt%2Egz

download.file("ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5FTNBC%5FSampleInfo%5FBCR%2Etxt%2Egz","GSE31519_TNBC_SampleInfo_BCR.txt.gz", mode = "wb")

# GSE31519_complete_dataset.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5Fcomplete%5Fdataset%2Etxt%2Egz#

download.file("ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5Fcomplete%5Fdataset%2Etxt%2Egz","GSE31519_complete_dataset.txt.gz", mode = "wb")

# This last file from the GEO database has a rather complicated structure with a gap of 23 columns 
#   since it also contains samples from the ArrayExpress database which they have moved to the end.
#   The file is used to rebuild the complete dataset of 579 samples here:

samples <- read.delim("GSE31519_complete_dataset.txt.gz", colClasses= "character", skip=1, nrows=1, header=FALSE)   # read the sample names from row-2
data <- read.delim("GSE31519_complete_dataset.txt.gz", skip=4,header=FALSE)  # read data starting row-5
colnames(data)=t(samples)       # add sample names from row-2 as column names
data=data[,!samples==""]        # remove blank gap-columns with no sample-title in row-2 from expr data
rm(samples)   # delete used variable
rownames(data)=data[,1]     # use first column with Affy IDs as rownames
cd.compl.579=data[,-1]    # remove first column with Affy IDs
rm(data)  # delete used variable
cd.compl.579=cd.compl.579[,sort(colnames(cd.compl.579))]   # sort columns by sample name

header.579 <- read.delim("GSE31519_TNBC_SampleInfo_BCR.txt.gz",colClasses = "character", row.names="SampleNames",h=T)   # all sample information

########################################################

##### Figure 1:  Selection of the TNBC finding cohort from multiple datasets based on dataset comparibility

#  Select the n=394 comparable TNBC samples from n=579 TNBC with normalized Affymetrix MAS5 data

#  Variables used:
#  cd.compl.579 ==> chip-data complete (data.frame, 579 samples in columns, 22283 probesets in rows)
#  header.579  ==> sample infos including dataset allocation (data.frame, 579 samples in columns)
#  n.probes ==> number of ProbeSets (rows in cd.compl.579)
#  n.col.cd ==> number of columns/samples in cd.compl.579
#  tcdm.579  ==> transposed chip data matrix
#  t.header.579 ==> transposed sample infos (data.frame)
#  tdsn.579  ==> transposed dataset allocation (numeric vector)
#

n.col.cd=ncol(cd.compl.579)
n.probes=nrow(cd.compl.579)
tdsn.579=as.numeric(t(header.579["datas_new09",]))   # transposed dataset vector
tcdm.579=t(cd.compl.579)   # transposed chip data matrix
t.header.579=as.data.frame(t(header.579))  # transposed sample info header as data.frame

ds.mean=by(tcdm.579, tdsn.579, colMeans)  # means of each probeset within 28 individual datasets, 
#  List is sorted by numeric dataset tdsn.579
tcdm.mean=apply(tcdm.579,2,mean)
#   generates named list of 22283 global means, which can be indexed by tcdm.mean[probes]
tcdm.stdev=apply(tcdm.579,2, sd)   # the same for the StdDev


# Calculate comparability metrics for the datasets:
# (calculate sum of squared differences of dataset-mean from total-mean for all probesets)
# Define variables
n.datas=length(ds.mean)   # number of datasets
diff.to.mean= matrix(0,nrow=n.probes,ncol=n.datas) # matrix of differences from mean
nrm.diff.to.mean= matrix(0,nrow=n.probes,ncol=n.datas) # matrix of NORMALIZED diff from mean

for (probes in 1:n.probes)   # loop for all probesets
{
  
  for (i in 1:n.datas)
    # calculate for each dataset diff from global-mean of all datasets
    # and save in matrix "diff.to.mean"
  {
    diff.to.mean[probes,i]=ds.mean[[i]][probes]- tcdm.mean[probes]
    # again the same but normalize by dividing through StdDev
    nrm.diff.to.mean[probes,i]=(ds.mean[[i]][probes]- tcdm.mean[probes]) / tcdm.stdev[probes]
  }
}
# calculate squares of differences
squ.diff=diff.to.mean^2
squ.nrm.diff= nrm.diff.to.mean^2
# sum of squared differences by column
sum.squ.diff=apply(na.omit(squ.diff),2,sum)
sum.squ.nrm.diff=apply(na.omit(squ.nrm.diff),2,sum)
#  important: these vectors are still sorted by numeric dataset tdsn.579 !

# summarize results
comparab=data.frame(sort(unique(tdsn.579)),sum.squ.diff,sum.squ.nrm.diff)
names(comparab)=c("dataset","sum.squ.diff","sum.squ.nrm.diff")
sort.comparab=comparab[order(comparab$sum.squ.nrm.diff),]

# integrate normalized comparab data in sample info in t.header.579
for (i in 1:n.col.cd) 
{t.header.579$comparab_nrm[i]= comparab$sum.squ.nrm.diff[comparab$dataset==tdsn.579[i]]}
# remove temporary variables:
rm(diff.to.mean, nrm.diff.to.mean, ds.mean,tcdm.mean, tcdm.stdev, squ.diff, squ.nrm.diff, sum.squ.diff, sum.squ.nrm.diff)

plot(sort(t.header.579$comparab_nrm),type="l")
abline(8000,0,col="red")


########################################################

# SELECT FINDING COHORT OF 394 TNBC

#  Select a subset of datasets with low comparability metric
# Select a subset of comparab by defining criteria:
compar.subset= subset(comparab, subset= sum.squ.nrm.diff < 8000)
# vector of corresponding datasets:
datas.subset=compar.subset$dataset
# generate logical vector FALSE/TRUE for the complete dataset of 579 TNBC:
subset.index.vector=(tdsn.579 %in% datas.subset)
# query selected samples from transposed chipdata matrix:
tcdm.394=tcdm.579[subset.index.vector , ]
# query  corresponding transposed dataset vector:
tdsn.394=tdsn.579[subset.index.vector]
# query selcted samples from NOT-transposed dataset (and corresponding header and t.header):
cd.compl.394= cd.compl.579[,subset.index.vector]
header.394= header.579[,subset.index.vector]
t.header.394= t.header.579[subset.index.vector,]

##### Figure 2: Analysis of a potential dataset bias among probesets

# Calculate Kruskal-Wallis statistics for all probesets according to their association with the dataset vector among the 394 TNBC samples:

# Variables used:    tcdm.394, tdsn.394, n.probes, cd.compl.394
# Newly generated variables:    kruskal.param

kruskal.result=kruskal.test(tcdm.394[,1],tdsn.394)  # Define variable as "list" by performing test once
kruskal.stat= matrix(0,nrow=n.probes)  # matrix for chi-statistics
kruskal.p=matrix(0,nrow=n.probes)  # matrix for p-values

# loop perfoming kruskal-test for each probeset (chip-data vs. dataset-vector)
# the results of each test are variables of type LIST, which are combined by indexing with [[i]]
for (i in 1:n.probes)
{
  kruskal.result[[i]]=kruskal.test(tcdm.394[,i], tdsn.394)   # results as LIST
  kruskal.stat[i]=kruskal.result[[i]]$statistic  # read statistics from LIST in matrix
  kruskal.p[i]=kruskal.result[[i]]$p.value   # read p-values from LIST in matrix
}
# summarize statistics and p-values and probeset names in one dataframe:
kruskal.param= data.frame(row.names=rownames(cd.compl.394),kruskal.stat,kruskal.p)  # dataframe with results
# remove temporary variables:
rm(kruskal.result, kruskal.stat, kruskal.p)

hist(kruskal.param$kruskal.stat,breaks=30,col="dark blue") # histogram of kruskal wallis stat for all probesets

# histogram of kruskal wallis stat for probes from hemoglobin metagene:

Hemoglobin.meta.probes=c("204419_x_at", "204848_x_at", "209116_x_at", "204018_x_at", "209458_x_at", "211745_x_at", "214414_x_at", "211696_x_at", "217232_x_at", "217414_x_at", "211699_x_at", "213515_x_at")
hist(kruskal.param[rownames(kruskal.param) %in% Hemoglobin.meta.probes,]$kruskal.stat,breaks=30,col="dark blue")


########################################################

